@@ -34,7 +34,10 @@
 # dependencies from prebuilt packages and use some existing system libraries.
 # =============================================================================
 
-SANDBOX_VERSION=20071205002
+./setup_binarykit.sh
+BKIT_VERSION="$(cat binary_kit_version)"
+
+PKG_DIR=$(pwd)/miro-binary-kit-osx-$BKIT_VERSION/sandbox
 
 # =============================================================================
 
@@ -62,14 +65,6 @@ done
 
 # -----------------------------------------------------------------------------
 
-if [ -d "../../../dtv-binary-kit-mac" ]; then
-    PKG_DIR=$(pushd ../../../dtv-binary-kit-mac/sandbox >/dev/null; pwd; popd  >/dev/null)
-else
-    echo "Could not find the required Mac binary kit which should be at $ROOT_DIR/dtv-binary-kit-mac"
-    echo "Please check it out first from the repository."
-    exit
-fi
-
 #SANDBOX_DIR=$ROOT_DIR/sandbox
 SANDBOX_DIR=/usr/local
 WORK_DIR=$SANDBOX_DIR/pkg
