@@ -2117,22 +2117,23 @@ def remove_column(cursor, table, *column_names):
     for sql in index_sql:
         cursor.execute(sql)
 
+def get_next_id(cursor):
+    """Calculate the next id to assign to new rows.
+
+    This will be 1 higher than the max id for all tables in the DB.
+    """
+    cursor.execute("SELECT name FROM sqlite_master "
+            "WHERE type='table' and name != 'dtv_variables'")
+    max_id = 0
+    for row in cursor.fetchall():
+        cursor.execute("SELECT MAX(id) from %s" % row[0])
+        max_id = max(max_id, cursor.fetchone()[0])
+    return max_id + 1
+
 def upgrade88(cursor):
     """Replace playlist.item_ids, with PlaylistItemMap objects."""
 
-    # get the last id in the database
-    max_id = 0
-    for table in ('channel_folder', 'playlist_folder', 'channel_guide',
-            'directory_feed_impl', 'directory_watch_feed_impl',
-            'remote_downloader', 'rss_feed_impl', 'feed',
-            'rss_multi_feed_impl', 'feed_impl', 'scraper_feed_impl',
-            'http_auth_password', 'search_downloads_feed_impl icon_cache',
-            'item', 'single_feed_impl', 'manual_feed_impl', 'taborder_order',
-            'theme_history', 'widgets_frontend_state', 'playlist'):
-        cursor.execute("SELECT MAX(id) from %s" % table)
-        max_id = max(max_id, cursor.fetchone()[0])
-
-    id_counter = itertools.count(max_id + 1)
+    id_counter = itertools.count(get_next_id(cursor))
 
     folder_count = {}
     for table_name in ('playlist_item_map', 'playlist_folder_item_map'):
@@ -2167,8 +2168,15 @@ def upgrade88(cursor):
     for row in list(cursor.execute(sql)):
         id, item_ids = row
         item_ids = eval(item_ids, {}, {})
+        this_folder_count = folder_count[id]
         for i, item_id in enumerate(item_ids):
-            count = folder_count[id][item_id]
+            try:
+                count = this_folder_count[item_id]
+            except KeyError:
+                # item_id is listed for this playlist folder, but none of it's
+                # child folders.  It's not clear how it happened, but forget
+                # about it.  (#12301)
+                continue
             cursor.execute("INSERT INTO playlist_folder_item_map "
                     "(id, item_id, playlist_id, position, count) "
                     "VALUES (?, ?, ?, ?, ?)",
@@ -2179,7 +2187,7 @@ def upgrade88(cursor):
 def upgrade89(cursor):
     """Set videoFilename column for downloaded items."""
     import datetime
-    from miro.plat.utils import FilenameType
+    from miro.plat.utils import filenameToUnicode
 
     # for Items, calculate from the downloader
     for row in cursor.execute("SELECT id, downloader_id FROM item "
@@ -2215,8 +2223,7 @@ def upgrade89(cursor):
         filename = status.get('filename')
         if (state in ('stopped', 'finished', 'uploading', 'uploading-paused')
                 and filename):
-            if FilenameType is not unicode:
-                filename = filename.decode('utf-8')
+            filename = filenameToUnicode(filename)
             cursor.execute("UPDATE item SET videoFilename=? WHERE id=?",
                 (filename, item_id))
     # for FileItems, just copy from filename
@@ -2328,6 +2335,7 @@ def upgrade98(cursor):
 def upgrade99(cursor):
     """Set the filename attribute for downloaded Item objects
     """
+    from miro.plat.utils import filenameToUnicode
     cursor.execute("SELECT id, status from remote_downloader "
             "WHERE state in ('stopped', 'finished', 'uploading', "
             "'uploading-paused')")
@@ -2337,6 +2345,7 @@ def upgrade99(cursor):
                 {'datetime': datetime, 'time': time})
         filename = status.get('filename')
         if filename:
+            filename = filenameToUnicode(filename)
             cursor.execute("UPDATE item SET filename=? WHERE downloader_id=?",
                     (filename, downloader_id))
 
@@ -2370,29 +2379,21 @@ def upgrade100(cursor):
     if count > 0:
         return
 
-    max_id = 0
-    for table in ('channel_folder', 'playlist_folder_item_map',
-                  'channel_guide', 'playlist_item_map', 'feed',
-                  'directory_feed_impl', 'remote_downloader',
-                  'directory_watch_feed_impl', 'rss_feed_impl',
-                  'rss_multi_feed_impl', 'scraper_feed_impl', 'feed_impl',
-                  'search_downloads_feed_impl', 'http_auth_password',
-                  'search_feed_impl', 'icon_cache', 'single_feed_impl',
-                  'item', 'taborder_order', 'manual_feed_impl', 'theme_history',
-                  'playlist', 'widgets_frontend_state', 'playlist_folder'):
-        cursor.execute("SELECT MAX(id) from %s" % table)
-        max_id = max(max_id, cursor.fetchone()[0])
+    next_id = get_next_id(cursor)
 
     cursor.execute("INSERT INTO channel_guide "
                    "(id, url, allowedURLs, updated_url, favicon, firstTime) VALUES (?, ?, ?, ?, ?, ?)",
-                   (max_id + 1, audio_guide_url, "[]", audio_guide_url, favicon_url, True))
+                   (next_id, audio_guide_url, "[]", audio_guide_url, favicon_url, True))
 
     # add the new Audio Guide to the site tablist
     cursor.execute('SELECT tab_ids FROM taborder_order WHERE type=?',('site',))
     row = cursor.fetchone()
     if row is not None:
-        tab_ids = eval_container(row[0])
-        tab_ids.append(max_id + 1)
+        try:
+            tab_ids = eval_container(row[0])
+        except StandardError:
+            tab_ids = []
+        tab_ids.append(next_id)
         cursor.execute('UPDATE taborder_order SET tab_ids=? WHERE type=?',
                        (repr(tab_ids), 'site'))
     else:
@@ -2410,7 +2411,12 @@ def upgrade101(cursor):
             "WHERE state = 'stopped'")
     for row in cursor.fetchall():
         id, status = row
-        status = eval_container(status)
+        try:
+            status = eval_container(status)
+        except StandardError:
+            # Not sure what to do here.  I think ignoring is not ideal, but
+            # won't result in anything too bad (BDK)
+            continue
         if status['endTime'] == status['startTime']:
             # For unfinished downloads, unset the filename which got set in
             # upgrade99
@@ -2472,3 +2478,33 @@ def upgrade103(cursor):
 def upgrade104(cursor):
     cursor.execute("UPDATE item SET seen=0 WHERE seen IS NULL")
     cursor.execute("UPDATE item SET keep=0 WHERE keep IS NULL")
+
+def upgrade105(cursor):
+    """Move metainfo and fastResumeData out of the status dict """
+    # create new colums
+    cursor.execute("ALTER TABLE remote_downloader ADD metainfo BLOB")
+    cursor.execute("ALTER TABLE remote_downloader ADD fast_resume_data BLOB")
+    # move things
+    cursor.execute("SELECT id, status FROM remote_downloader")
+    for row in cursor.fetchall():
+        id, status_repr = row
+        try:
+            status = eval_container(status_repr)
+        except StandardError:
+            status = {}
+        metainfo = status.pop('metainfo', None)
+        fast_resume_data = status.pop('fastResumeData', None)
+        new_status = repr(status)
+        if metainfo is not None:
+            metainfo_value = buffer(metainfo)
+        else:
+            metainfo_value = None
+        if fast_resume_data is not None:
+            fast_resume_data_value = buffer(fast_resume_data)
+        else:
+            fast_resume_data_value = None
+        cursor.execute("UPDATE remote_downloader "
+                "SET status=?, metainfo=?, fast_resume_data=? "
+                "WHERE id=?",
+                (new_status, metainfo_value, fast_resume_data_value, id))
+
