@@ -51,7 +51,7 @@ from miro import config
 from miro import prefs
 
 from miro.dl_daemon import command, daemon
-from miro.util import checkF, checkU, stringify
+from miro.util import checkF, checkU, stringify, MAX_TORRENT_SIZE
 from miro.plat.utils import get_available_bytes_for_movies, utf8_to_filename
 
 chatter = True
@@ -335,6 +335,8 @@ class DownloadStatusUpdater:
     happen often enough to cause CPU problems.
     """
 
+    # setting this to 2 to prevent the "downloader playing catch-up"
+    # issue where suddenly a bunch of updates happen all at once.
     UPDATE_CLIENT_INTERVAL = 2
 
     def __init__(self):
@@ -1086,23 +1088,45 @@ class BTDownloader(BGDownloader):
                     raise RuntimeError()
                 name = metainfo['info']['name']
             except RuntimeError:
-                self.handleError(_("Corrupt Torrent"),
-                                 _("The torrent file at %(url)s was not valid") % {"url": stringify(self.url)})
+                self.handleCorruptTorrent()
                 return
             self.shortFilename = utf8_to_filename(name)
             self.pickInitialFilename(suffix="", torrent=True)
         self.updateClient()
         self._resumeTorrent()
 
+    def handleCorruptTorrent(self):
+        self.handleError(_("Corrupt Torrent"),
+                         _("The torrent file at %(url)s was not valid") % {"url": stringify(self.url)})
+
     def handleMetainfo(self, metainfo):
         self.metainfo = metainfo
         self.gotMetainfo()
 
+    def _cleanup_description_vars(self):
+        del self.description_chunks
+        del self.description_size
+        del self.description_client
+
+    def on_description_data(self, data):
+        self.description_chunks.append(data)
+        self.description_size += len(data)
+        if (self.description_size > MAX_TORRENT_SIZE or
+                self.description_chunks[0][0] != 'd'):
+            # Bailout if we get too much data or it doesn't begin with "d"
+            # (see #12301 for details)
+            self.description_client.cancel()
+            self.handleCorruptTorrent()
+            self._cleanup_description_vars()
+
     def onDescriptionDownload(self, info):
-        self.handleMetainfo(info['body'])
+        body = ''.join(self.description_chunks)
+        self.handleMetainfo(body)
+        self._cleanup_description_vars()
 
     def onDescriptionDownloadFailed(self, exception):
         self.handleNetworkError(exception)
+        self._cleanup_description_vars()
 
     def getMetainfo(self):
         if self.metainfo is None:
@@ -1125,8 +1149,12 @@ class BTDownloader(BGDownloader):
 
                 self.handleMetainfo(metainfo)
             else:
-                httpclient.grabURL(self.get_url(), self.onDescriptionDownload,
-                        self.onDescriptionDownloadFailed)
+                self.description_chunks = []
+                self.description_size = 0
+                self.description_client = httpclient.grabURL(self.get_url(),
+                        self.onDescriptionDownload,
+                        self.onDescriptionDownloadFailed,
+                        bodyDataCallback=self.on_description_data)
         else:
             self.gotMetainfo()
                 
