@@ -37,6 +37,7 @@ from miro.util import get_torrent_info_hash, returnsUnicode, checkU, returnsFile
 # from miro import app
 from miro import config
 from miro import dialogs
+from miro import eventloop
 from miro import httpclient
 from miro import models
 from miro import prefs
@@ -126,6 +127,7 @@ class RemoteDownloader(DDBObject):
         self.main_item_id = None
         self.dlid = generate_dlid()
         self.status = {}
+        self.metainfo = self.fast_resume_data = None
         self.state = u'downloading'
         if contentType is None:
             # HACK:  Some servers report the wrong content-type for torrent
@@ -138,6 +140,7 @@ class RemoteDownloader(DDBObject):
         self.deleteFiles = True
         self.channelName = channelName
         self.manualUpload = False
+        self._save_later_dc = None
         if contentType is None:
             self.contentType = u""
         else:
@@ -175,6 +178,37 @@ class RemoteDownloader(DDBObject):
         if needsSignalItem:
             for item in self.itemList:
                 item.signal_change(needsSave=False)
+        if needsSave:
+            self._cancel_save_later()
+
+
+    def _save_later(self):
+        """Save the remote downloader at some point in the future.
+
+        This is used to handle the fact that remote downloaders are updated
+        often, but those updates are usually just the status dict, which is
+        never used for SELECT statements.  Continually saving those changes
+        to disk is just a waste of time and IO.
+
+        Instead, we schedule the save to happen sometime in the future.  When
+        miro quits, we call the module-level function run_delayed_saves(),
+        which makes sure any pending objects are saved to disk.
+        """
+        if self._save_later_dc is None:
+            self._save_later_dc = eventloop.addTimeout(15,
+                    self._save_now, "Delayed RemoteDownloader save")
+
+    def _save_now(self):
+        """If _save_later() was called and we haven't saved the downloader to
+        disk, do it now.
+        """
+        if self.idExists() and self._save_later_dc is not None:
+            self.signal_change()
+
+    def _cancel_save_later(self):
+        if self._save_later_dc is not None:
+            self._save_later_dc.cancel()
+            self._save_later_dc = None
 
     def on_content_type(self, info):
         if not self.idExists():
@@ -241,14 +275,20 @@ class RemoteDownloader(DDBObject):
         self = get_downloader_by_dlid(dlid=data['dlid'])
         # print data
         if self is not None:
+            # FIXME - this should get fixed.
+            metainfo = data.pop('metainfo', None)
+            fast_resume_data = data.pop('fastResumeData', None)
+            current = (self.status, self.metainfo, self.fast_resume_data)
+            new = (data, metainfo, fast_resume_data)
             try:
-                if self.status == data:
+                if current == new:
                     return
             except Exception, e:
                 # This is a known bug with the way we used to save fast resume
                 # data
                 print "WARNING exception when comparing status: %s" % e
 
+            old_state = self.state
             wasFinished = self.isFinished()
             old_filename = self.get_filename()
             self.before_changing_status()
@@ -258,7 +298,14 @@ class RemoteDownloader(DDBObject):
             if data.has_key('activity') and data['activity']:
                 data['activity'] = _(data['activity'])
 
-            self.status = data
+            # only set attributes if something's changed.  This makes our
+            # UPDATE statments contain less data
+            if data != self.status:
+                self.status = data
+            if metainfo != self.metainfo:
+                self.metainfo = metainfo
+            if fast_resume_data != self.fast_resume_data:
+                self.fast_resume_data = fast_resume_data
             self._recalc_state()
 
             # Store the time the download finished
@@ -271,7 +318,12 @@ class RemoteDownloader(DDBObject):
             if self.get_state() == u'uploading' and not self.manualUpload and self.getUploadRatio() > 1.5:
                 self.stopUpload()
 
-            self.signal_change(needsSignalItem = needsSignalItem)
+            self.signal_change(needsSignalItem=needsSignalItem,
+                    needsSave=False)
+            if self.state == old_state:
+                self._save_later()
+            else:
+                self.signal_change()
             if finished:
                 for item in self.itemList:
                     item.on_download_finished()
@@ -545,6 +597,7 @@ URL was %s""" % self.url
         return self.status.get('filename', FilenameType(''))
 
     def setup_restored(self):
+        self._save_later_dc = None
         self.deleteFiles = True
         self.itemList = []
         if self.dlid == 'noid':
@@ -583,8 +636,11 @@ URL was %s""" % self.url
                 self.run_downloader()
         else:
             _downloads[self.dlid] = self
-            c = command.RestoreDownloaderCommand(RemoteDownloader.dldaemon, 
-                                                 self.status)
+            dler_status = self.status
+            dler_status['metainfo'] = self.metainfo
+            dler_status['fastResumeData'] = self.fast_resume_data
+            c = command.RestoreDownloaderCommand(RemoteDownloader.dldaemon,
+                                                 dler_status)
             c.send()
 
     def startUpload(self):
@@ -705,10 +761,17 @@ class DownloadDaemonStarter(object):
             downloader.restartIfNeeded()
 
     def shutdown(self, callback):
+        self.shutdown_callback = callback
         if not self.started:
-            callback()
+            self._on_shutdown()
         else:
-            RemoteDownloader.dldaemon.shutdown_downloader_daemon(callback=callback)
+            RemoteDownloader.dldaemon.shutdown_downloader_daemon(
+                    callback=self._on_shutdown)
+
+    def _on_shutdown(self):
+        ensure_downloaders_saved()
+        self.shutdown_callback()
+        del self.shutdown_callback
 
 def init_controller():
     """Intializes the download daemon controller.
@@ -774,3 +837,12 @@ def get_downloader_for_item(item):
             return RemoteDownloader(url, item, u'application/x-bittorrent', channelName=channelName)
     else:
         return RemoteDownloader(url, item, channelName=channelName)
+
+def ensure_downloaders_saved():
+    """Make sure any RemoteDownloaders with pending changes get saved.
+
+    For performance reasons, RemoteDownloader objects don't save to disk every
+    time their status dict is updated.
+    """
+    for downloader in RemoteDownloader.make_view():
+        downloader._save_now()
