@@ -26,6 +26,7 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
+import logging
 import os
 
 from miro import app
@@ -38,19 +39,20 @@ from miro import messages
 from miro.plat.frontends.widgets import timer
 from miro.plat.frontends.widgets import widgetset
 from miro.frontends.widgets.displays import VideoDisplay
-#from miro.frontends.widgets.displays import AudioDisplay
-#from miro.frontends.widgets.displays import ExternalVideoDisplay
 
 class PlaybackManager (signals.SignalEmitter):
     
     def __init__(self):
         signals.SignalEmitter.__init__(self)
+        self.player = None
         self.video_display = None
+        self.removing_video_display = False
         self.detached_window = None
         self.previous_left_width = 0
         self.previous_left_widget = None
         self.is_fullscreen = False
         self.is_playing = False
+        self.is_playing_audio = False
         self.is_paused = False
         self.is_suspended = False
         self.open_successful = False
@@ -120,8 +122,8 @@ class PlaybackManager (signals.SignalEmitter):
 
     def set_volume(self, volume):
         self.volume = volume
-        if self.video_display is not None:
-            self.video_display.set_volume(volume)
+        if self.player is not None:
+            self.player.set_volume(volume)
     
     def set_presentation_mode(self, mode):
         self.presentation_mode = mode
@@ -137,22 +139,22 @@ class PlaybackManager (signals.SignalEmitter):
             self.pause()
     
     def start_with_items(self, item_infos, presentation_mode='fit-to-bounds'):
-        self.playlist = item_infos
+        if self.is_playing:
+            # If we get here and we are already playing something it most
+            # certainly means that the currently playing item is an audio one
+            # which leaves the UI free to start another item, so stop first.
+            self.stop()
+        self.playlist = []
+        for info in item_infos:
+            if not info.is_container_item:
+                self.playlist.append(info)
+            else:
+                playlables = [i for i in info.children if i.is_playable]
+                self.playlist.extend(playlables)
         self.position = 0
         self._calc_id_to_position()
         self.presentation_mode = presentation_mode
         self._start_tracking_items()
-        if not self.is_playing:
-            self.video_display = VideoDisplay()
-            self.video_display.connect('removed', self.on_display_removed)
-            self.video_display.connect('cant-play', self._on_cant_play)
-            self.video_display.connect('ready-to-play', self._on_ready_to_play)
-            if config.get(prefs.PLAY_DETACHED):
-                self.prepare_detached_playback()
-            else:
-                self.prepare_attached_playback()
-            self.is_playing = True
-            app.menu_manager.handle_playing_selection()
         self._play_current()
         if self.presentation_mode != 'fit-to-bounds':
             self.fullscreen()
@@ -208,7 +210,6 @@ class PlaybackManager (signals.SignalEmitter):
         self.detached_window.close(False)
         self.detached_window.destroy()
         self.detached_window = None
-        app.tab_list_manager.update_menus()
     
     def schedule_update(self):
         def notify_and_reschedule():
@@ -226,36 +227,37 @@ class PlaybackManager (signals.SignalEmitter):
             self.update_timeout = None
 
     def notify_update(self):
-        if self.video_display is not None:
-            elapsed = self.video_display.get_elapsed_playback_time()
-            total = self.video_display.get_total_playback_time()
+        if self.player is not None:
+            elapsed = self.player.get_elapsed_playback_time()
+            total = self.player.get_total_playback_time()
             self.emit('playback-did-progress', elapsed, total)
 
     def on_display_removed(self, display):
-        self.stop()
+        if not self.removing_video_display:
+            self.stop()
 
     def play(self):
-        duration = self.video_display.get_total_playback_time()
+        duration = self.player.get_total_playback_time()
         self.emit('will-play', duration)
         resume_time = self.playlist[self.position].resume_time
         if (config.get(prefs.RESUME_VIDEOS_MODE)
                and resume_time > 10
                and not self.is_paused):
-            self.video_display.play_from_time(resume_time)
+            self.player.play_from_time(resume_time)
         else:
-            self.video_display.play()
+            self.player.play()
         self.notify_update()
         self.schedule_update()
         self.is_paused = False
         self.is_suspended = False
-        app.menu_manager.set_play_pause("pause")
+        app.menu_manager.update_menus()
 
     def pause(self):
         if self.is_playing:
             self.emit('will-pause')
-            self.video_display.pause()
+            self.player.pause()
             self.is_paused = True
-            app.menu_manager.set_play_pause("play")
+            app.menu_manager.update_menus()
 
     def fullscreen(self):
         if not self.is_playing:
@@ -272,28 +274,36 @@ class PlaybackManager (signals.SignalEmitter):
         self.cancel_update_timer()
         self.cancel_mark_as_watched()
         self.is_playing = False
+        self.is_playing_audio = False
         self.is_paused = False
         self.emit('will-stop')
+        if self.player is not None:
+            self.player.stop()
+            self.player = None
         if self.video_display is not None:
-            self.video_display.stop()
-            if self.detached_window is not None:
-                self.video_display.cleanup()
-                self.finish_detached_playback()
-            else:
-                self.finish_attached_playback()
+            self.remove_video_display()
+            self.video_display = None
         self.is_fullscreen = False
         self.previous_left_widget = None
-        self.video_display = None
         self.position = 0
         self.playlist = None
         self.emit('did-stop')
 
+    def remove_video_display(self):
+        self.removing_video_display = True
+        if self.detached_window is not None:
+            self.video_display.cleanup()
+            self.finish_detached_playback()
+        else:
+            self.finish_attached_playback()
+        self.removing_video_display = False
+
     def update_current_resume_time(self, resume_time=-1):
         if not self.open_successful:
             return
         if config.get(prefs.RESUME_VIDEOS_MODE):
             if resume_time == -1:
-                resume_time = self.video_display.get_elapsed_playback_time()
+                resume_time = self.player.get_elapsed_playback_time()
         else:
             resume_time = 0
         id = self.playlist[self.position].id
@@ -301,22 +311,23 @@ class PlaybackManager (signals.SignalEmitter):
 
     def set_playback_rate(self, rate):
         if self.is_playing:
-            self.video_display.set_playback_rate(rate)
+            self.player.set_playback_rate(rate)
 
     def suspend(self):
         if self.is_playing and not self.is_paused:
-            self.video_display.pause()
+            self.player.pause()
         self.is_suspended = True
     
     def resume(self):
         if self.is_playing and not self.is_paused:
-            self.video_display.play()
+            self.player.play()
         self.is_suspended = False
 
     def seek_to(self, progress):
-        self.video_display.seek_to(progress)
-        total = self.video_display.get_total_playback_time()
-        self.emit('playback-did-progress', progress * total, total)
+        self.player.seek_to(progress)
+        total = self.player.get_total_playback_time()
+        if total is not None:
+            self.emit('playback-did-progress', progress * total, total)
 
     def on_movie_finished(self):
         self.update_current_resume_time(0)
@@ -344,14 +355,65 @@ class PlaybackManager (signals.SignalEmitter):
             return self.playlist[self.position]
         return None
 
+    def _setup_player(self, item_info, volume):
+        def _handle_successful_sniff(item_type):
+            self._finish_setup_player(item_info, item_type, volume)
+        def _handle_unsuccessful_sniff():
+            self._finish_setup_player(item_info, "unplayable", volume)
+        widgetset.get_item_type(item_info, _handle_successful_sniff, _handle_unsuccessful_sniff)
+    
+    def _finish_setup_player(self, item_info, item_type, volume):
+        if item_type == 'audio':
+            if self.is_playing and self.video_display is not None:
+                # if we were previously playing a video get rid of the video
+                # display first
+                self.player.stop()
+                self.player = None
+                self.remove_video_display()
+                self.video_display = None
+            if self.player is None or not self.is_playing:
+                self._build_audio_player(item_info, volume)
+            self.is_playing = True
+            self.player.setup(item_info, volume)
+        elif item_type in ('video', 'unplayable'):
+            # We send items with type 'other' to the video display to be able 
+            # to open them using the 'play externally' display - luc.
+            if self.is_playing and self.video_display is None:
+                # if we were previously playing an audio file, stop.
+                self.stop()
+                return
+            if self.video_display is None or not self.is_playing:
+                self._build_video_player(item_info, volume)
+            self.is_playing = True
+            self.video_display.setup(item_info, volume)
+            if self.detached_window is not None:
+                self.detached_window.set_title(item_info.name)
+        app.menu_manager.update_menus()
+
+    def _build_video_player(self, item_info, volume):
+        self.player = widgetset.VideoPlayer()
+        self.video_display = VideoDisplay(self.player)
+        self.video_display.connect('removed', self.on_display_removed)
+        self.video_display.connect('cant-play', self._on_cant_play)
+        self.video_display.connect('ready-to-play', self._on_ready_to_play)
+        if config.get(prefs.PLAY_DETACHED):
+            self.prepare_detached_playback()
+        else:
+            self.prepare_attached_playback()
+        self.is_playing_audio = False
+
+    def _build_audio_player(self, item_info, volume):
+        self.player = widgetset.AudioPlayer()
+        self.player.connect('cant-play', self._on_cant_play)
+        self.player.connect('ready-to-play', self._on_ready_to_play)
+        self.is_playing_audio = True
+
     def _select_current(self):
-        self.emit('selecting-file')
         volume = config.get(prefs.VOLUME_LEVEL)
         item_info = self.playlist[self.position]
+        self.emit('selecting-file', item_info)
         self.open_successful = False
-        self.video_display.setup(item_info, volume)
-        if self.detached_window is not None:
-            self.detached_window.set_title(item_info.name)
+        self._setup_player(item_info, volume)
 
     def _play_current(self, new_position=None, save_resume_time=True):
         """If you pass in new_position, then this will attempt to play
@@ -363,22 +425,23 @@ class PlaybackManager (signals.SignalEmitter):
 
         self.cancel_update_timer()
         self.cancel_mark_as_watched()
-
         if (0 <= new_position < len(self.playlist)):
             self.position = new_position
             if self.is_playing:
-                self.video_display.stop()
+                self.player.stop(True)
             self._select_current()
         else:
             self.stop(save_resume_time)
 
-    def _on_ready_to_play(self, video_display):
+    def _on_ready_to_play(self, obj):
         self.open_successful = True
         self.schedule_mark_as_watched(self.playlist[self.position].id)
         self.play()
 
-    def _on_cant_play(self, video_display):
+    def _on_cant_play(self, obj):
         self.emit('cant-play-file')
+        if isinstance(obj, widgetset.AudioPlayer):
+            self.play_next_movie(False)
 
     def play_next_movie(self, save_resume_time=True):
         self.play_from_position(self.position + 1, save_resume_time)
@@ -397,10 +460,10 @@ class PlaybackManager (signals.SignalEmitter):
             self._play_current(new_position, save_resume_time)
 
     def skip_forward(self):
-        self.video_display.skip_forward()
+        self.player.skip_forward()
 
     def skip_backward(self):
-        self.video_display.skip_backward()
+        self.player.skip_backward()
 
     def toggle_fullscreen(self):
         if self.is_fullscreen:
@@ -424,7 +487,7 @@ class PlaybackManager (signals.SignalEmitter):
             self.switch_to_detached_playback()
         else:
             self.switch_to_attached_playback()
-        app.menu_manager.handle_playing_selection()
+        app.menu_manager.update_menus()
             
     def switch_to_attached_playback(self):
         self.cancel_update_timer()
@@ -445,6 +508,7 @@ class DetachedWindow(widgetset.Window):
         widgetset.Window.__init__(self, title, rect)
         self.closing = False
         self.stop_on_close = True
+        self.connect_menu_keyboard_shortcuts()
 
     def close(self, stop_playback=True):
         if not self.closing:
@@ -460,20 +524,33 @@ class DetachedWindow(widgetset.Window):
     def do_key_press(self, key, mods):
         if handle_key_press(key, mods):
             return True
-        if menubar.MOD in mods and key == 't':
-            app.playback_manager.toggle_detached_mode()
-            return True
         return False
 
 def handle_key_press(key, mods):
     """Handle a playback key press events """
 
     if len(mods) != 0:
+        if set([menubar.MOD, menubar.SHIFT]) == mods:
+            if key in ('>', '.'): # OS X sends '.', GTK sends '>'
+                app.widgetapp.on_forward_clicked()
+                return True
+            elif key in ('<', ','): # OS X sends ',', GTK sends '<'
+                app.widgetapp.on_previous_clicked()
+                return True
         return False
 
-    if key == menubar.ESCAPE and app.playback_manager.is_fullscreen:
-        app.widgetapp.on_fullscreen_clicked()
-        return True
+    if key == menubar.ESCAPE:
+        if app.playback_manager.is_fullscreen:
+            app.widgetapp.on_fullscreen_clicked()
+            return True
+        else:
+            app.widgetapp.on_stop_clicked()
+            return True
+
+    # anything after this point doesn't work when we're playing
+    # audio
+    if app.playback_manager.is_playing_audio:
+        return False
 
     if key == menubar.RIGHT_ARROW:
         app.widgetapp.on_skip_forward()
