@@ -63,6 +63,12 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 	boost::tie(protocol, auth, hostname, port, path, error)
 		= parse_url_components(url);
 
+	int default_port = protocol == "https" ? 443 : 80;
+
+	// keep ourselves alive even if the callback function
+	// deletes this object
+	boost::shared_ptr<http_connection> me(shared_from_this());
+
 	if (error)
 	{
 		callback(asio::error::socket_type_not_supported);
@@ -99,7 +105,9 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 	else
 	{
 		headers << "GET " << path << " HTTP/1.0\r\n"
-			"Host:" << hostname << ":" << port << "\r\n";
+			"Host: " << hostname;
+		if (port != default_port) headers << ":" << to_string(port).elems;
+		headers << "\r\n";
 	}
 
 	if (!auth.empty())
@@ -139,6 +147,10 @@ void http_connection::start(std::string const& hostname, std::string const& port
 	m_read_pos = 0;
 	m_priority = prio;
 
+	// keep ourselves alive even if the callback function
+	// deletes this object
+	boost::shared_ptr<http_connection> me(shared_from_this());
+
 	if (ec)
 	{
 		callback(ec);
@@ -202,6 +214,10 @@ void http_connection::on_connect_timeout()
 	if (m_connection_ticket > -1) m_cc.done(m_connection_ticket);
 	m_connection_ticket = -1;
 
+	// keep ourselves alive even if the callback function
+	// deletes this object
+	boost::shared_ptr<http_connection> me(shared_from_this());
+
 	if (!m_endpoints.empty())
 	{
 		error_code ec;
@@ -263,6 +279,8 @@ void http_connection::on_resolve(error_code const& e
 {
 	if (e)
 	{
+		boost::shared_ptr<http_connection> me(shared_from_this());
+
 		callback(e);
 		close();
 		return;
@@ -328,6 +346,7 @@ void http_connection::on_connect(error_code const& e)
 	} 
 	else
 	{ 
+		boost::shared_ptr<http_connection> me(shared_from_this());
 		callback(e);
 		close();
 	}
@@ -365,6 +384,7 @@ void http_connection::on_write(error_code const& e)
 {
 	if (e)
 	{
+		boost::shared_ptr<http_connection> me(shared_from_this());
 		callback(e);
 		close();
 		return;
@@ -399,6 +419,10 @@ void http_connection::on_read(error_code const& e
 		TORRENT_ASSERT(m_download_quota >= 0);
 	}
 
+	// keep ourselves alive even if the callback function
+	// deletes this object
+	boost::shared_ptr<http_connection> me(shared_from_this());
+
 	// when using the asio SSL wrapper, it seems like
 	// we get the shut_down error instead of EOF
 	if (e == asio::error::eof || e == asio::error::shut_down)
@@ -539,7 +563,7 @@ void http_connection::on_read(error_code const& e
 	m_sock.async_read_some(asio::buffer(&m_recvbuffer[0] + m_read_pos
 		, amount_to_read)
 		, bind(&http_connection::on_read
-		, shared_from_this(), _1, _2));
+		, me, _1, _2));
 }
 
 void http_connection::on_assign_bandwidth(error_code const& e)
