@@ -267,15 +267,19 @@ PlaybackItems = [
 ]
 
 HelpItems = [
-    MenuItem(_("_About Miro"), "About", ()),
-    MenuItem(_("_Donate"), "Donate", ()),
-    MenuItem(_("_Help"), "Help", (Key(F1),)),
-    Separator(),
-    MenuItem(_("Diagnostics"), "Diagnostics", ()),
-    MenuItem(_("Report a _Bug"), "ReportBug", ()),
-    MenuItem(_("_Translate"), "Translate", ()),
-    MenuItem(_("_Planet Miro"), "Planet", ()),
-]
+    MenuItem(_("_About %(name)s", {'name': config.get(prefs.SHORT_APP_NAME)}), "About", ())]
+if config.get(prefs.DONATE_URL):
+   HelpItems.append(MenuItem(_("_Donate"), "Donate", ()))
+if config.get(prefs.HELP_URL):
+   HelpItems.append(MenuItem(_("_Help"), "Help", (Key(F1),)))
+HelpItems.extend([Separator(),
+                   MenuItem(_("Diagnostics"), "Diagnostics", ())])
+if config.get(prefs.BUG_REPORT_URL):
+    HelpItems.append(MenuItem(_("Report a _Bug"), "ReportBug", ()))
+if config.get(prefs.TRANSLATE_URL):
+    HelpItems.append(MenuItem(_("_Translate"), "Translate", ()))
+if config.get(prefs.PLANET_URL):
+   HelpItems.append(MenuItem(_("_Planet Miro"), "Planet", ()))
 
 # FIXME - move this to platform-specific code
 if platform == "gtk-x11":
