@@ -142,18 +142,22 @@ class LiveStorage:
             self._all_schemas.append(oschema)
             for klass in oschema.ddb_object_classes():
                 self._schema_map[klass] = oschema
+                for field_name, schema_item in oschema.fields:
+                    klass.track_attribute_changes(field_name)
         self._converter = SQLiteConverter()
 
         if not db_existed:
             self._init_database()
 
     def open_connection(self):            
+        logging.info("opening database %s", self.path)
         self.connection = sqlite3.connect(self.path,
                 isolation_level=None,
                 detect_types=sqlite3.PARSE_DECLTYPES)
         self.cursor = self.connection.cursor()
 
     def close(self):
+        logging.info("closing database")
         if self._dc:
             self._dc.cancel()
             self._dc = None
@@ -275,8 +279,8 @@ class LiveStorage:
         del self._object_map[obj.id]
         self._ids_loaded.remove(obj.id)
 
-    def update_obj(self, obj):
-        """Update a DDBObject on disk."""
+    def insert_obj(self, obj):
+        """Add a new DDBObject to disk."""
 
         obj_schema = self._schema_map[obj.__class__]
         column_names = []
@@ -292,12 +296,39 @@ class LiveStorage:
                 raise
             values.append(self._converter.to_sql(obj_schema, name,
                 schema_item, value))
-        sql = "REPLACE INTO %s (%s) VALUES(%s)" % (obj_schema.table_name,
+        sql = "INSERT INTO %s (%s) VALUES(%s)" % (obj_schema.table_name,
                 ', '.join(column_names),
                 ', '.join('?' for i in xrange(len(column_names))))
         self._execute(sql, values, is_update=True)
         self.remember_object(obj)
 
+    def update_obj(self, obj):
+        """Update a DDBObject on disk."""
+
+        obj_schema = self._schema_map[obj.__class__]
+        setters = []
+        values = []
+        for name, schema_item in obj_schema.fields:
+            if (isinstance(schema_item, schema.SchemaSimpleItem) and
+                    name not in obj.changed_attributes):
+                continue
+            setters.append('%s=?' % name)
+            value = getattr(obj, name)
+            try:
+                schema_item.validate(value)
+            except schema.ValidationError:
+                if util.chatter:
+                    logging.warn("error validating %s for %s", name, obj)
+                raise
+            values.append(self._converter.to_sql(obj_schema, name,
+                schema_item, value))
+        obj.reset_changed_attributes()
+        if values:
+            sql = "UPDATE %s SET %s WHERE id=%s" % (obj_schema.table_name,
+                    ', '.join(setters), obj.id)
+            self._execute(sql, values)
+        self.remember_object(obj)
+
     def remove_obj(self, obj):
         """Remove a DDBObject from disk."""
 
@@ -394,17 +425,37 @@ class LiveStorage:
 
     def _restore_object_from_row(self, schema, db_row):
         restored_data = {}
+        columns_to_update = []
+        values_to_update = []
         for (name, schema_item), value in \
                 itertools.izip(schema.fields, db_row):
             try:
                 value = self._converter.from_sql(schema, name, schema_item,
                         value)
-            except:
-                if util.chatter:
-                    logging.warn("error converting %s (%r)", name,
-                            value)
-                raise
+            except StandardError:
+                handler = self._converter.get_malformed_data_handler(schema,
+                        name, schema_item, value)
+                if handler is None:
+                    if util.chatter:
+                        logging.warn("error converting %s (%r)", name, value)
+                    raise
+                try:
+                    value = handler(value)
+                except StandardError:
+                    if util.chatter:
+                        logging.warn("error converting %s (%r)", name, value)
+                    raise
+                columns_to_update.append(name)
+                values_to_update.append(self._converter.to_sql(schema, name,
+                    schema_item, value))
             restored_data[name] = value
+        if columns_to_update:
+            # We are using some values that are different than what's stored
+            # in disk.  Update the database to make things match.
+            setters = ['%s=?' % c for c in columns_to_update]
+            sql = "UPDATE %s SET %s WHERE id=%s" % (schema.table_name,
+                    ', '.join(setters), restored_data['id'])
+            self._execute(sql, values_to_update)
         klass = schema.get_ddb_class(restored_data)
         return klass(restored_data=restored_data)
 
@@ -656,15 +707,14 @@ class SQLiteConverter(object):
             return None
         converter = self._from_sql_converters.get(schema_item.__class__,
                 self._null_convert)
-        try:
-            return converter(value)
-        except Exception, e:
-            handler_name = 'handle_malformed_%s' % name
-            if hasattr(schema, handler_name):
-                handler = getattr(schema, handler_name)
-            else:
-                raise
-            return handler(value)
+        return converter(value)
+
+    def get_malformed_data_handler(self, schema, name, schema_item, value):
+        handler_name = 'handle_malformed_%s' % name
+        if hasattr(schema, handler_name):
+            return getattr(schema, handler_name)
+        else:
+            return None
 
     def _unicode_to_filename(self, value):
         return value.encode('utf-8')
