@@ -33,7 +33,6 @@
 # front end
 from miro.test.unicodetest import *
 
-from miro.test.databasetest import *
 from miro.test.fasttypestest import *
 from miro.test.schematest import *
 from miro.test.storedatabasetest import *
@@ -48,11 +47,13 @@ from miro.test.feedparsertest import *
 from miro.test.parseurltest import *
 from miro.test.utiltest import *
 from miro.test.playlisttest import *
-from miro.test.databaseupgradetest import *
 from miro.test.signalstest import *
 from miro.test.messagetest import *
 from miro.test.strippertest import *
 from miro.test.xhtmltest import *
 from miro.test.gtcachetest import *
+from miro.test.iconcachetest import *
+from miro.test.databasetest import *
+from miro.test.itemtest import *
 
 import miro.test.bmachinetest
